unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs,
  Vcl.StdCtrls, Vcl.ComCtrls, Vcl.ExtCtrls;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Label1: TLabel;
    UpDown1: TUpDown;
    ENumCharts: TEdit;
    Panel2: TPanel;
    BStartStop: TButton;
    CBDrawAll: TCheckBox;
    CBFastPen: TCheckBox;
    EMaxPoints: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    EScrollPoints: TEdit;
    procedure ENumChartsChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure BStartStopClick(Sender: TObject);
    procedure CBDrawAllClick(Sender: TObject);
    procedure CBFastPenClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses VCLTee.Chart, VCLTee.TeEngine, VCLTee.Series, VCLTee.TeeTools,
     VCLTee.TeeConst;

var Charts  : TList;
    Stopped : Boolean;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Caption:=TeeMsg_Version;
  Charts:=TList.Create;

  UpDown1.Min:=1;
  UpDown1.Max:=3;

  EMaxPoints.NumbersOnly:=true;
  EScrollPoints.NumbersOnly:=true;

  Stopped:=True;
end;

procedure TForm1.BStartStopClick(Sender: TObject);
var ScrollPoints, MaxPoints: Integer;

  // Adds a new random point to Series
  Procedure RealTimeAdd(Series: TChartSeries);
  begin
    if Series.Count=0 then  // First random point
       Series.AddXY(1,Random(10000))
    else
       // Next random point
       Series.AddXY(Series.XValues.Last+1,Series.YValues.Last+Random(10)-4.5);
  end;

  // When the chart is filled with points, this procedure
  // deletes and scrolls points to the left.
  Procedure DoScrollPoints(Chart: TChart);
  var tmp,tmpMin,tmpMax : Double;
  begin
    // Delete multiple points with a single call.
    // Much faster than deleting points using a loop.
    Chart[0].Delete(0,ScrollPoints);

    // Scroll horizontal bottom axis
    tmp:=Chart[0].XValues.Last;
    Chart.Axes.Bottom.SetMinMax(tmp-MaxPoints+ScrollPoints,tmp);

    // Scroll vertical left axis
    tmpMin:=Chart[0].YValues.MinValue;
    tmpMax:=Chart[0].YValues.MaxValue;

    with Chart.Axes.Left do
      if not Automatic then
         SetMinMax(tmpMin-tmpMin/5,tmpMax+tmpMax/5);

    // Do chart repaint after deleting and scrolling
    Application.ProcessMessages;
  end;

var i: Integer;
    chart: TChart;
begin
  if Stopped then
  begin
    // Start loop...
    BStartStop.Caption:='&Stop';
    ENumCharts.Enabled:=False;
    EMaxPoints.Enabled:=False;
    EScrollPoints.Enabled:=False;
    CBDrawAll.Enabled:=False;
    CBFastPen.Enabled:=False;

    // Prepare variables
    MaxPoints:=StrToInt(EMaxPoints.Text);
    ScrollPoints:=StrToInt(EScrollPoints.Text);

    for i:=0 to Charts.Count-1 do
    begin
      chart:=Charts[i];
      chart.Axes.Bottom.SetMinMax(1,MaxPoints);
      chart[0].Clear;
      Application.ProcessMessages;
      Stopped:=False;
    end;

    // Start loop
    while not Stopped do
    begin
      for i:=0 to Charts.Count-1 do
      begin
        chart:=Charts[i];

        // Add one more point
        RealTimeAdd(chart[0]);

        // Delete and Scroll points to the left
        if chart[0].Count>Pred(MaxPoints) then
           DoScrollPoints(chart);
      end;
    end;
  end
  else
  begin
    // Finish
    Stopped:=True;
    BStartStop.Caption:='&Start';
    ENumCharts.Enabled:=True;
    EMaxPoints.Enabled:=True;
    EScrollPoints.Enabled:=True;
    CBDrawAll.Enabled:=True;
    CBFastPen.Enabled:=True;
  end;
end;

procedure TForm1.CBDrawAllClick(Sender: TObject);
var i: Integer;
begin
  for i:=0 to Charts.Count-1 do
    TFastLineSeries(TChart(Charts[i])[0]).DrawAllPoints:=CBDrawAll.Checked;
end;

procedure TForm1.CBFastPenClick(Sender: TObject);
var i: Integer;
begin
  for i:=0 to Charts.Count-1 do
    TFastLineSeries(TChart(Charts[i])[0]).FastPen:=CBFastPen.Checked;
end;

procedure TForm1.ENumChartsChange(Sender: TObject);
var chart: TChart;
begin
  while Charts.Count <> UpDown1.Position do
  begin
    if Charts.Count < UpDown1.Position then
    begin
      chart:=TChart.Create(Self);
      Charts.Add(chart);
      with chart do
      begin
        Parent:=Panel2;
        Align:=alTop;
        Height:=130;
        Legend.Visible:=False;
        Title.Visible:=False;
        ClipPoints:=False;
        View3D:=False;
        Axes.Bottom.Labels:=false;
        Tools.Add(TRepaintMonitor);
      end;

      with chart.AddSeries(TFastLineSeries) as TFastLineSeries do
      begin
        DrawAllPoints:=CBDrawAll.Checked;
        FastPen:=CBFastPen.Checked;
      end;
    end
    else
    begin
      chart:=Charts[Charts.Count-1];
      Charts.Delete(Charts.Count-1);
      chart.Destroy;
    end;
  end;
end;

end.
